@@ -45,6 +45,7 @@ import (
 
 const (
 	x86ContentRootDir = "/opt/openshift/operator/x86_64"
+	armContentRootDir = "/opt/openshift/operator/aarch64"
 	ppcContentRootDir = "/opt/openshift/operator/ppc64le"
 	zContentRootDir   = "/opt/openshift/operator/s390x"
 	installtypekey    = "keyForInstallTypeField"
@@ -81,15 +82,16 @@ func NewSamplesOperatorHandler(kubeconfig *restclient.Config,
 
 	h.imageclientwrapper = &defaultImageStreamClientWrapper{h: h, lister: listers.ImageStreams}
 	h.templateclientwrapper = &defaultTemplateClientWrapper{h: h, lister: listers.Templates}
+	h.configmapclientwrapper = &defaultConfigMapClientWrapper{h: h, lister: listers.ConfigMaps}
 	h.cvowrapper = operatorstatus.NewClusterOperatorHandler(h.configclient)
 
 	h.skippedImagestreams = make(map[string]bool)
 	h.skippedTemplates = make(map[string]bool)
 
-	h.CreateDefaultResourceIfNeeded(nil)
-
 	h.imagestreamFile = make(map[string]string)
 	h.templateFile = make(map[string]string)
+	h.imagestreatagToImage = make(map[string]string)
+	h.CreateDefaultResourceIfNeeded(nil)
 
 	h.imagestreamRetry = make(map[string]metav1.Time)
 
@@ -113,8 +115,9 @@ type Handler struct {
 	coreclient   *corev1client.CoreV1Client
 	configclient *configv1client.ConfigV1Client
 
-	imageclientwrapper    ImageStreamClientWrapper
-	templateclientwrapper TemplateClientWrapper
+	imageclientwrapper     ImageStreamClientWrapper
+	templateclientwrapper  TemplateClientWrapper
+	configmapclientwrapper ConfigMapClientWrapper
 
 	crdlister        configv1lister.ConfigLister
 	streamlister     imagev1lister.ImageStreamNamespaceLister
@@ -129,8 +132,9 @@ type Handler struct {
 	skippedTemplates    map[string]bool
 	skippedImagestreams map[string]bool
 
-	imagestreamFile map[string]string
-	templateFile    map[string]string
+	imagestreamFile      map[string]string
+	templateFile         map[string]string
+	imagestreatagToImage map[string]string
 
 	imagestreamRetry map[string]metav1.Time
 
@@ -147,35 +151,34 @@ type Handler struct {
 // - cfg: return this if we want to check the status of a prior upsert
 // - filePath: used to the caller to look up the image content for the upsert
 // - doUpsert: whether to do the upsert of not ... not doing the upsert optionally triggers the need for checking status of prior upsert
-// - updateCfgOnly: if we want to update the cfg without any upsert attempts or processing
 // - err: if a problem occurred getting the Config, we return the error to bubble up and initiate a retry
-func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[string]string, deleted bool) (*v1.Config, string, bool, bool, error) {
+func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[string]string, deleted bool) (*v1.Config, string, bool, error) {
 	cfg, err := h.crdwrapper.Get(v1.ConfigName)
 	if cfg == nil || err != nil {
 		// if not found, then this also would mean a deletion event
 		if kerrors.IsNotFound(err) {
 			logrus.Printf("Received watch event %s but not upserting since deletion of the Config is in progress", kind+"/"+name)
-			return nil, "", false, false, nil
+			return nil, "", false, nil
 		}
 		logrus.Printf("Received watch event %s but not upserting since not have the Config yet: %#v %#v", kind+"/"+name, err, cfg)
-		return nil, "", false, false, err
+		return nil, "", false, err
 	}
 
 	if cfg.DeletionTimestamp != nil {
 		// we do no return the cfg in this case because we do not want to bother with any progress tracking
 		logrus.Printf("Received watch event %s but not upserting since deletion of the Config is in progress", kind+"/"+name)
 		// note, the imagestream watch cache gets cleared once the deletion/finalizer processing commences
-		return nil, "", false, false, nil
+		return nil, "", false, nil
 	}
 
 	// we do not return the cfg in these cases because we do not want to bother with any progress tracking
 	switch cfg.Spec.ManagementState {
 	case operatorsv1api.Removed:
 		logrus.Debugf("Not upserting %s/%s event because operator is in removed state and image changes are not in progress", kind, name)
-		return nil, "", false, false, nil
+		return nil, "", false, nil
 	case operatorsv1api.Unmanaged:
 		logrus.Debugf("Not upserting %s/%s event because operator is in unmanaged state and image changes are not in progress", kind, name)
-		return nil, "", false, false, nil
+		return nil, "", false, nil
 	}
 
 	filePath := ""
@@ -198,32 +201,17 @@ func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[strin
 			logrus.Printf("watch stream event %s not part of operators inventory", name)
 			// we now have cases where sample providers are deleting entire imagestreams;
 			// let's make sure there are no stale entries with inprogress / importerror
-			processing := util.Condition(cfg, v1.ImageChangesInProgress)
-			needToUpdate := util.NameInReason(cfg, processing.Reason, name)
-			if needToUpdate {
-				now := kapis.Now()
-				processing.Reason = util.ClearNameInReason(cfg, processing.Reason, name)
-				logrus.Debugf("processing reason now %s", processing.Reason)
-				if len(strings.TrimSpace(processing.Reason)) == 0 {
-					logrus.Println("The last in progress imagestream has completed (removed imagestream check)")
-					processing.Status = corev1.ConditionFalse
-					processing.Reason = ""
-					processing.LastTransitionTime = now
-				}
-				processing.LastUpdateTime = now
-				util.ConditionUpdate(cfg, processing)
+			_, err := h.configmapclientwrapper.Get(name)
+			if err != nil && kerrors.IsNotFound(err) {
+				// ConfigMap should only exist if the imagestream has an error
+				return nil, "", false, nil
 			}
-			importErrors := util.Condition(cfg, v1.ImportImageErrorsExist)
-			needToUpdate2 := util.NameInReason(cfg, importErrors.Reason, name)
-			if needToUpdate2 {
-				h.mapsMutex.Lock()
-				defer h.mapsMutex.Unlock()
-				h.clearStreamFromImportError(name, importErrors, cfg)
+			if err != nil {
+				// GET errors indicate a potential issue with the apiserver, return error and try again
+				return nil, "", false, err
 			}
-			if needToUpdate || needToUpdate2 {
-				return cfg, "", false, true, nil
-			}
-			return nil, "", false, false, nil
+			err = h.configmapclientwrapper.Delete(name)
+			return nil, "", false, err
 		}
 		_, skipped = h.skippedImagestreams[name]
 	case "template":
@@ -232,7 +220,7 @@ func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[strin
 			// in the case of templates we can just ignore content from prior releases that is no longer
 			// part of the current release
 			logrus.Printf("watch template event %s not part of operators inventory", name)
-			return nil, "", false, false, nil
+			return nil, "", false, nil
 		}
 		_, skipped = h.skippedTemplates[name]
 	}
@@ -240,21 +228,20 @@ func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[strin
 	if skipped {
 		logrus.Printf("watch event %s in skipped list for %s", name, kind)
 		// but return cfg to potentially toggle pending/import error condition
-		return cfg, "", false, false, nil
+		return cfg, "", false, nil
 	}
 
-	if deleted && (kind == "template" || cache.UpsertsAmount() == 0) {
+	if deleted { //&& (kind == "template" || cache.UpsertsAmount() == 0) {
 		logrus.Printf("going to recreate deleted managed sample %s/%s", kind, name)
-		return cfg, filePath, true, false, nil
+		return cfg, filePath, true, nil
 	}
 
-	if util.ConditionFalse(cfg, v1.ImageChangesInProgress) &&
-		(util.ConditionTrue(cfg, v1.MigrationInProgress) || h.version != cfg.Status.Version) {
+	if h.shouldSetVersion(cfg) {
 		// we have gotten events for items early in the migration list but we have not
 		// finished processing the list
 		// avoid (re)upsert, but check import status
 		logrus.Printf("watch event for %s/%s while migration in progress, image in progress is false; will not update sample because of this event", kind, name)
-		return cfg, "", false, false, nil
+		return cfg, "", false, nil
 	}
 
 	if annotations != nil {
@@ -263,11 +250,11 @@ func (h *Handler) prepSamplesWatchEvent(kind, name string, annotations map[strin
 		if ok && isv == h.version {
 			logrus.Debugf("Not upserting %s/%s cause operator version matches", kind, name)
 			// but return cfg to potentially toggle pending condition
-			return cfg, "", false, false, nil
+			return cfg, "", false, nil
 		}
 	}
 
-	return cfg, filePath, true, false, nil
+	return cfg, filePath, true, nil
 }
 
 func (h *Handler) GoodConditionUpdate(cfg *v1.Config, newStatus corev1.ConditionStatus, conditionType v1.ConfigConditionType) {
@@ -323,6 +310,8 @@ func (h *Handler) updateCfgArch(cfg *v1.Config) *v1.Config {
 		cfg.Spec.Architectures = append(cfg.Spec.Architectures, v1.PPCArchitecture)
 	case strings.Contains(runtime.GOARCH, v1.S390Architecture):
 		cfg.Spec.Architectures = append(cfg.Spec.Architectures, v1.S390Architecture)
+	case strings.Contains(runtime.GOARCH, v1.ARMArchitecture):
+		cfg.Spec.Architectures = append(cfg.Spec.Architectures, v1.ARMArchitecture)
 	case strings.Contains(runtime.GOARCH, v1.AMDArchitecture):
 		fallthrough
 	case strings.Contains(runtime.GOARCH, v1.X86Architecture):
@@ -434,6 +423,13 @@ func (h *Handler) CreateDefaultResourceIfNeeded(cfg *v1.Config) (*v1.Config, err
 		cfg.Kind = "Config"
 		cfg.APIVersion = v1.GroupName + "/" + v1.Version
 		cfg = h.updateCfgArch(cfg)
+
+		// build file maps and create configmaps with imagestreamtag to image mappings
+		err := h.buildFileMaps(cfg, true)
+		if err != nil {
+			return nil, err
+		}
+
 		switch {
 		// TODO as we gain content for non x86 platforms we can remove the nonx86 check
 		case util.IsUnsupportedArch(cfg):
@@ -535,8 +531,6 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 		}
 	}
 
-	cache.ClearUpsertsCache()
-
 	tempList, err := h.templateclientwrapper.List(iopts)
 	if err != nil && !kerrors.IsNotFound(err) {
 		logrus.Warnf("Problem listing openshift templates on Config delete: %#v", err)
@@ -560,6 +554,23 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 		}
 	}
 
+	cmList, err := h.configmapclientwrapper.List()
+	if err != nil && !kerrors.IsNotFound(err) {
+		logrus.Warnf("Problem listing sample operator config maps on Config delete: %v", err.Error())
+		return err
+	} else {
+		for _, cm := range cmList {
+			if cm.Name == util.IST2ImageMap {
+				continue
+			}
+			err := h.configmapclientwrapper.Delete(cm.Name)
+			if err != nil && !kerrors.IsNotFound(err) {
+				logrus.Warnf("Problem deleting samples operator config map %s on Config delete: %v", cm.Name, err.Error())
+				return err
+			}
+		}
+	}
+
 	// FYI we no longer delete the credential because the payload imagestreams like cli, must-gather that
 	// this operator initially installs via its manifest, but does not manage, needs the pull image secret
 
@@ -568,6 +579,13 @@ func (h *Handler) CleanUpOpenshiftNamespaceOnDelete(cfg *v1.Config) error {
 
 func (h *Handler) Handle(event util.Event) error {
 	switch event.Object.(type) {
+	case *corev1.ConfigMap:
+		cm, _ := event.Object.(*corev1.ConfigMap)
+		if cm.Name == util.IST2ImageMap {
+			return nil
+		}
+		return h.processImageCondition()
+
 	case *imagev1.ImageStream:
 		is, _ := event.Object.(*imagev1.ImageStream)
 		if is.Namespace != "openshift" {
@@ -596,11 +614,11 @@ func (h *Handler) Handle(event util.Event) error {
 		// and event delete flag true
 		if event.Deleted {
 			logrus.Info("A previous delete attempt has been successfully completed")
-			h.cvowrapper.UpdateOperatorStatus(cfg, true, h.tbrCheckFailed)
+			h.cvowrapper.UpdateOperatorStatus(cfg, true, h.tbrCheckFailed, h.activeImageStreams())
 			return nil
 		}
 		if cfg.DeletionTimestamp != nil {
-			h.cvowrapper.UpdateOperatorStatus(cfg, true, h.tbrCheckFailed)
+			h.cvowrapper.UpdateOperatorStatus(cfg, true, h.tbrCheckFailed, h.activeImageStreams())
 			// before we kick off the delete cycle though, we make sure a prior creation
 			// cycle is not still in progress, because we don't want the create adding back
 			// in things we just deleted ... if an upsert is still in progress, return an error;
@@ -614,7 +632,7 @@ func (h *Handler) Handle(event util.Event) error {
 			}
 
 			// nuke any registered upserts
-			cache.ClearUpsertsCache()
+			//cache.ClearUpsertsCache()
 
 			if h.NeedsFinalizing(cfg) {
 				// so we initiate the delete and set exists to false first, where if we get
@@ -673,15 +691,15 @@ func (h *Handler) Handle(event util.Event) error {
 		// Every time we see a change to the Config object, update the ClusterOperator status
 		// based on the current conditions of the Config.
 		cfg = h.refetchCfgMinimizeConflicts(cfg)
-		err := h.cvowrapper.UpdateOperatorStatus(cfg, false, h.tbrCheckFailed)
+		err := h.cvowrapper.UpdateOperatorStatus(cfg, false, h.tbrCheckFailed, h.activeImageStreams())
 		if err != nil {
 			logrus.Errorf("error updating cluster operator status: %v", err)
 			return err
 		}
 
 		cfg = h.refetchCfgMinimizeConflicts(cfg)
-		doit, cfgUpdate, err := h.ProcessManagementField(cfg)
-		if !doit || err != nil {
+		updateStatusManagementState, cfgUpdate, err := h.ProcessManagementField(cfg)
+		if !updateStatusManagementState || err != nil {
 			if err != nil || cfgUpdate {
 				// flush status update
 				dbg := fmt.Sprintf("process mgmt update spec %s status %s", string(cfg.Spec.ManagementState), string(cfg.Status.ManagementState))
@@ -711,6 +729,7 @@ func (h *Handler) Handle(event util.Event) error {
 
 		if len(cfg.Spec.Architectures) > 0 &&
 			cfg.Spec.Architectures[0] != v1.AMDArchitecture &&
+			cfg.Spec.Architectures[0] != v1.ARMArchitecture &&
 			cfg.Spec.Architectures[0] != v1.X86Architecture &&
 			cfg.Spec.Architectures[0] != v1.S390Architecture &&
 			cfg.Spec.Architectures[0] != v1.PPCArchitecture {
@@ -720,17 +739,15 @@ func (h *Handler) Handle(event util.Event) error {
 		h.buildSkipFilters(cfg)
 		configChanged := false
 		configChangeRequiresUpsert := false
-		configChangeRequiresImportErrorUpdate := false
 		registryChanged := false
 		unskippedStreams := map[string]bool{}
 		unskippedTemplates := map[string]bool{}
 		if cfg.Spec.ManagementState == cfg.Status.ManagementState {
 			cfg = h.refetchCfgMinimizeConflicts(cfg)
-			configChanged, configChangeRequiresUpsert, configChangeRequiresImportErrorUpdate, registryChanged, unskippedStreams, unskippedTemplates = h.VariableConfigChanged(cfg)
-			logrus.Debugf("config changed %v upsert needed %v import error upd needed %v exists/true %v progressing/false %v op version %s status version %s",
+			configChanged, configChangeRequiresUpsert, registryChanged, unskippedStreams, unskippedTemplates = h.VariableConfigChanged(cfg)
+			logrus.Debugf("config changed %v upsert needed %v exists/true %v progressing/false %v op version %s status version %s",
 				configChanged,
 				configChangeRequiresUpsert,
-				configChangeRequiresImportErrorUpdate,
 				util.ConditionTrue(cfg, v1.SamplesExist),
 				util.ConditionFalse(cfg, v1.ImageChangesInProgress),
 				h.version,
@@ -741,15 +758,7 @@ func (h *Handler) Handle(event util.Event) error {
 				util.ConditionTrue(cfg, v1.SamplesExist) &&
 				util.ConditionFalse(cfg, v1.ImageChangesInProgress) &&
 				h.version == cfg.Status.Version {
-				logrus.Debugf("At steady state: config the same and exists is true, in progress false, and version correct")
-
-				// once the status version is in sync, we can turn off the migration condition
-				if util.ConditionTrue(cfg, v1.MigrationInProgress) {
-					h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.MigrationInProgress)
-					dbg := " turn migration off"
-					logrus.Printf("CRDUPDATE %s", dbg)
-					return h.crdwrapper.UpdateStatus(cfg, dbg)
-				}
+				logrus.Printf("At steady state: config the same and exists is true, in progress false, and version correct")
 
 				// migration inevitably means we need to refresh the file cache as samples are added and
 				// deleted between releases, so force file map building
@@ -785,22 +794,7 @@ func (h *Handler) Handle(event util.Event) error {
 		}
 
 		cfg = h.refetchCfgMinimizeConflicts(cfg)
-		if util.ConditionFalse(cfg, v1.MigrationInProgress) &&
-			len(cfg.Status.Version) > 0 &&
-			h.version != cfg.Status.Version {
-			logrus.Printf("Undergoing migration from %s to %s", cfg.Status.Version, h.version)
-			h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.MigrationInProgress)
-			dbg := "turn migration on"
-			logrus.Printf("CRDUPDATE %s", dbg)
-			return h.crdwrapper.UpdateStatus(cfg, dbg)
-		}
-
-		cfg = h.refetchCfgMinimizeConflicts(cfg)
-		if !configChanged &&
-			util.ConditionTrue(cfg, v1.SamplesExist) &&
-			util.ConditionFalse(cfg, v1.ImageChangesInProgress) &&
-			util.Condition(cfg, v1.MigrationInProgress).LastUpdateTime.Before(&util.Condition(cfg, v1.ImageChangesInProgress).LastUpdateTime) &&
-			h.version != cfg.Status.Version {
+		if !configChanged && h.shouldSetVersion(cfg) {
 			if util.ConditionTrue(cfg, v1.ImportImageErrorsExist) {
 				logrus.Printf("An image import error occurred applying the latest configuration on version %s; this operator will periodically retry the import, or an administrator can investigate and remedy manually", h.version)
 			}
@@ -809,16 +803,6 @@ func (h *Handler) Handle(event util.Event) error {
 			dbg := "upd status version"
 			logrus.Printf("CRDUPDATE %s", dbg)
 			return h.crdwrapper.UpdateStatus(cfg, dbg)
-			/*if cfg.ConditionFalse(cfg, v1.ImportImageErrorsExist) {
-				cfg.Status.Version = h.version
-				logrus.Printf("The samples are now at version %s", cfg.Status.Version)
-				logrus.Println("CRDUPDATE upd status version")
-				return h.crdwrapper.UpdateStatus(cfg)
-			}
-			logrus.Printf("An image import error occurred applying the latest configuration on version %s, problem resolution needed", h.version
-			return nil
-
-			*/
 		}
 
 		if len(cfg.Spec.Architectures) == 0 {
@@ -836,21 +820,26 @@ func (h *Handler) Handle(event util.Event) error {
 			h.setSampleManagedLabelToFalse("imagestream", name)
 		}
 
-		// this boolean is driven by VariableConfigChanged based on comparing spec/status skip lists and
-		// cross referencing with any image import errors
-		if configChangeRequiresImportErrorUpdate && !configChangeRequiresUpsert {
-			dbg := "config change did not require upsert but did change import errors"
-			logrus.Printf("CRDUPDATE %s", dbg)
-			return h.crdwrapper.UpdateStatus(cfg, dbg)
-		}
-
 		if configChanged && !configChangeRequiresUpsert && util.ConditionTrue(cfg, v1.SamplesExist) {
 			dbg := "bypassing upserts for non invasive config change after initial create"
 			logrus.Printf("CRDUPDATE %s", dbg)
 			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
-		if !util.ConditionTrue(cfg, v1.ImageChangesInProgress) {
+		if !util.ConditionTrue(cfg, v1.SamplesExist) ||
+			!util.ConditionFalse(cfg, v1.ImageChangesInProgress) ||
+			h.version != cfg.Status.Version ||
+			configChanged ||
+			updateStatusManagementState {
+			logrus.Infof("ENTERING UPSERT / STEADY STATE PATH ExistTrue %v ImageInProgressFalse %v VersionOK %v ConfigChanged %v ManagementStateChanged %v",
+				util.ConditionTrue(cfg, v1.SamplesExist),
+				util.ConditionFalse(cfg, v1.ImageChangesInProgress),
+				h.version == cfg.Status.Version,
+				configChanged,
+				updateStatusManagementState)
+			cfg.Status.Version = h.version
+			h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.SamplesExist)
+			h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.ImageChangesInProgress)
 			// pass in true to force rebuild of maps, which we do here because at this point
 			// we have taken on some form of config change
 			err = h.buildFileMaps(cfg, true)
@@ -861,6 +850,28 @@ func (h *Handler) Handle(event util.Event) error {
 			h.upsertInProgress = true
 			turnFlagOff := func(h *Handler) { h.upsertInProgress = false }
 			defer turnFlagOff(h)
+
+			for isName := range h.imagestreamFile {
+				_, skipped := h.skippedImagestreams[isName]
+				unskipping := len(unskippedStreams) > 0
+				_, unskipped := unskippedStreams[isName]
+				if (unskipping && !unskipped) || skipped {
+					continue
+				}
+
+				cm := &corev1.ConfigMap{
+					ObjectMeta: metav1.ObjectMeta{
+						Name:      isName,
+						Namespace: v1.OperatorNamespace,
+					},
+					Data: map[string]string{},
+				}
+				_, err = h.configmapclientwrapper.Create(cm)
+				if err != nil && !kerrors.IsAlreadyExists(err) {
+					return err
+				}
+			}
+
 			abortForDelete, err := h.createSamples(cfg, true, registryChanged, unskippedStreams, unskippedTemplates)
 			// we prioritize enabling delete vs. any error processing from createSamples (though at the moment that
 			// method only returns nil error when it returns true for abortForDelete) as a subsequent delete's processing will
@@ -886,114 +897,17 @@ func (h *Handler) Handle(event util.Event) error {
 				}
 				return err
 			}
-			now := kapis.Now()
-			cfg = h.refetchCfgMinimizeConflicts(cfg)
-			progressing := util.Condition(cfg, v1.ImageChangesInProgress)
-			progressing.Reason = ""
-			progressing.Message = ""
-			for isName := range h.imagestreamFile {
-				_, skipped := h.skippedImagestreams[isName]
-				unskipping := len(unskippedStreams) > 0
-				_, unskipped := unskippedStreams[isName]
-				if unskipping && !unskipped {
-					continue
-				}
-				if !util.NameInReason(cfg, progressing.Reason, isName) && !skipped {
-					progressing.Reason = progressing.Reason + isName + " "
-				}
-			}
-			if len(progressing.Reason) > 0 {
-				progressing.LastUpdateTime = now
-				progressing.LastTransitionTime = now
-				logrus.Debugf("Handle changing processing from false to true")
-				progressing.Status = corev1.ConditionTrue
-			} else {
-				logrus.Debugln("there are no imagestreams available for import")
-				// see if there are unskipped templates, to set SamplesExists to true
-				if !util.ConditionTrue(cfg, v1.SamplesExist) {
-					templatesProcessed := false
-					for tName := range h.templateFile {
-						_, skipped := h.skippedTemplates[tName]
-						if skipped {
-							continue
-						}
-						templatesProcessed = true
-						break
-					}
-					if templatesProcessed {
-						cfg = h.refetchCfgMinimizeConflicts(cfg)
-						h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.SamplesExist)
-						dbg := "exist true update templates only"
-						logrus.Printf("CRDUPDATE %s", dbg)
-						return h.crdwrapper.UpdateStatus(cfg, dbg)
-					}
-				}
-				return nil
-			}
-			logrus.Debugf("Handle Reason field set to %s", progressing.Reason)
-			util.ConditionUpdate(cfg, progressing)
 
 			// now that we employ status subresources, we can't populate
 			// the conditions on create; so we do initialize here, which is our "step 1"
 			// of the "make a change" flow in our state machine
 			cfg = h.initConditions(cfg)
 
-			dbg := "progressing true update"
+			dbg := "samples upserted; set clusteroperator ready, steady state"
 			logrus.Printf("CRDUPDATE %s", dbg)
 			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
 
-		if !util.ConditionTrue(cfg, v1.SamplesExist) {
-			cfg = h.refetchCfgMinimizeConflicts(cfg)
-			h.GoodConditionUpdate(cfg, corev1.ConditionTrue, v1.SamplesExist)
-			dbg := "exist true update"
-			logrus.Printf("CRDUPDATE %s", dbg)
-			return h.crdwrapper.UpdateStatus(cfg, dbg)
-		}
-
-		// it is possible that all the cached imagestream events show that
-		// the image imports are complete; hence we would not get any more
-		// events until the next relist to clear out in progress; so let's
-		// cycle through them here now
-		if cache.AllUpsertEventsArrived() && util.ConditionTrue(cfg, v1.ImageChangesInProgress) {
-			keysToClear := []string{}
-			anyChange := false
-			ac := false
-			cfg = h.refetchCfgMinimizeConflicts(cfg)
-			for key, is := range cache.GetUpsertImageStreams() {
-				if is == nil {
-					// never got update, refetch
-					var e error
-					is, e = h.imageclientwrapper.Get(key)
-					if e != nil {
-						keysToClear = append(keysToClear, key)
-						anyChange = true
-						continue
-					}
-				}
-				cfg, _, ac = h.processImportStatus(is, cfg)
-				anyChange = anyChange || ac
-			}
-			for _, key := range keysToClear {
-				cache.RemoveUpsert(key)
-				util.ClearNameInReason(cfg, util.Condition(cfg, v1.ImageChangesInProgress).Reason, key)
-				util.ClearNameInReason(cfg, util.Condition(cfg, v1.ImportImageErrorsExist).Reason, key)
-			}
-			if len(strings.TrimSpace(util.Condition(cfg, v1.ImageChangesInProgress).Reason)) == 0 {
-				h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.ImageChangesInProgress)
-				logrus.Println("The last in progress imagestream has completed (config event loop)")
-			}
-			if anyChange {
-				dbg := "updating in progress after examining cached imagestream events"
-				logrus.Printf("CRDUPDATE %s", dbg)
-				err = h.crdwrapper.UpdateStatus(cfg, dbg)
-				if err == nil && util.ConditionFalse(cfg, v1.ImageChangesInProgress) {
-					// only clear out cache if we got the update through
-					cache.ClearUpsertsCache()
-				}
-				return err
-			}
-		}
 	}
 	return nil
 }
@@ -1066,9 +980,6 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 		}
 
 		if _, isok := h.skippedImagestreams[imagestream.Name]; !isok {
-			if updateIfPresent {
-				cache.AddUpsert(imagestream.Name)
-			}
 			imagestreams = append(imagestreams, imagestream)
 		}
 
@@ -1092,16 +1003,13 @@ func (h *Handler) createSamples(cfg *v1.Config, updateIfPresent, registryChanged
 
 		err = h.upsertImageStream(imagestream, is, cfg)
 		if err != nil {
-			if updateIfPresent {
-				cache.RemoveUpsert(imagestream.Name)
-			}
 			return false, err
 		}
 	}
 
 	// if after initial startup, and not migration, the only cfg change was changing the registry, since that does not impact
 	// the templates, we can move on
-	if len(unskippedTemplates) == 0 && registryChanged && util.ConditionTrue(cfg, v1.SamplesExist) && util.ConditionFalse(cfg, v1.MigrationInProgress) {
+	if len(unskippedTemplates) == 0 && registryChanged && h.sufficientSteadyStateForAnOperator(cfg) {
 		return false, nil
 	}
 
@@ -1166,3 +1074,116 @@ func GetNamespace() string {
 	b, _ := ioutil.ReadFile("/var/run/secrets/kubernetes.io/serviceaccount/" + corev1.ServiceAccountNamespaceKey)
 	return string(b)
 }
+
+func (h *Handler) shouldSetProgressingFalse(cfg *v1.Config) bool {
+	present := h.numOfManagedImageStreamsPresent()
+	managed := h.numOfManagedImageStreams()
+	if present < managed {
+		logrus.Printf("shouldSetProgressingFalse: only %d of the %d imagestreams exist", present, managed)
+	}
+	return util.ConditionTrue(cfg, v1.SamplesExist) &&
+		util.ConditionTrue(cfg, v1.ImageChangesInProgress) &&
+		(len(h.activeImageStreams()) == 0 || util.ConditionTrue(cfg, v1.ImportImageErrorsExist)) &&
+		!h.upsertInProgress &&
+		present >= managed
+}
+
+func (h *Handler) shouldSetVersion(cfg *v1.Config) bool {
+	return util.ConditionTrue(cfg, v1.SamplesExist) &&
+		util.ConditionFalse(cfg, v1.ImageChangesInProgress) &&
+		!h.upsertInProgress &&
+		h.version != cfg.Status.Version &&
+		h.areStreamsAtCorrectVersion()
+}
+
+func (h *Handler) sufficientSteadyStateForAnOperator(cfg *v1.Config) bool {
+	return util.ConditionTrue(cfg, v1.SamplesExist) &&
+		util.ConditionFalse(cfg, v1.ImageChangesInProgress) &&
+		h.version == cfg.Status.Version
+}
+
+func (h *Handler) areStreamsAtCorrectVersion() bool {
+	for key, _ := range h.imagestreamFile {
+		// remember, our get wrapper goes through the lister, so we are only hitting the controller cache
+		skipped, ok := h.skippedImagestreams[key]
+		if skipped && ok {
+			continue
+		}
+		is, err := h.imageclientwrapper.Get(key)
+		if err != nil {
+			if !h.upsertInProgress {
+				// not all imagestreams may be created yet during initial or bulk creations
+				logrus.Warningf("areStreamsAtCorrectVersion: get of stream %s got error: %s", key, err.Error())
+			}
+			return false
+		}
+		if is == nil {
+			if !h.upsertInProgress {
+				// not all imagestreams may be created yet during initial or bulk creations
+				logrus.Warningf("areStreamsAtCorrectVersion: got nil for stream %s but no error", key)
+			}
+			return false
+		}
+		if is.Annotations == nil {
+			logrus.Warningf("areStreamsAtCorrectVersion: stream %s has no annotation map", key)
+			return false
+		}
+		isv, ok := is.Annotations[v1.SamplesVersionAnnotation]
+		if !ok {
+			logrus.Warningf("areStreamsAtCorrectVersion: stream %s does not have version annotation", key)
+			return false
+		}
+		if isv != h.version {
+			logrus.Warningf("areStreamsAtCorrectVersion: stream %s is at version %s instead of %s", key, isv, h.version)
+			return false
+		}
+	}
+	logrus.Printf("areStreamsAtCorrectVersion: all streams are now at the correct version %s", h.version)
+	return true
+}
+
+func (h *Handler) numOfManagedImageStreams() int {
+	rc := len(h.imagestreamFile) - len(h.skippedImagestreams)
+	return rc
+}
+
+func (h *Handler) numOfManagedImageStreamsPresent() int {
+	rc := 0
+	for key, _ := range h.imagestreamFile {
+		// remember, our get wrapper goes through the lister, so we are only hitting the controller cache
+		skipped, ok := h.skippedImagestreams[key]
+		if skipped && ok {
+			continue
+		}
+		is, err := h.imageclientwrapper.Get(key)
+		if err != nil {
+			if !h.upsertInProgress {
+				// not all imagestreams may be created yet during initial or bulk creations
+				logrus.Warningf("numOfManagedImageStreamsPresent: get of stream %s got error: %s", key, err.Error())
+			}
+			continue
+		}
+		if is == nil {
+			if !h.upsertInProgress {
+				// not all imagestreams may be created yet during initial or bulk creations
+				logrus.Warningf("numOfManagedImageStreamsPresent: got nil for stream %s but no error", key)
+			}
+			continue
+		}
+		if is.Labels == nil {
+			logrus.Warningf("numOfManagedImageStreamsPresent: stream %s has no labels map", key)
+			continue
+		}
+		managed, ok := is.Labels[v1.SamplesManagedLabel]
+		if !ok {
+			logrus.Warningf("numOfManagedImageStreamsPresent: stream %s does not have managed label", key)
+			continue
+		}
+		if managed != "true" {
+			logrus.Warningf("numOfManagedImageStreamsPresent: stream %s has managed value of %s but is not in internal skip list", key, managed)
+			continue
+		}
+		rc++
+	}
+	return rc
+}
