@@ -11,23 +11,22 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	kerrors "k8s.io/apimachinery/pkg/api/errors"
 	kapis "k8s.io/apimachinery/pkg/apis/meta/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	imagev1 "github.com/openshift/api/image/v1"
 	v1 "github.com/openshift/api/samples/v1"
 
-	"github.com/openshift/cluster-samples-operator/pkg/cache"
 	"github.com/openshift/cluster-samples-operator/pkg/metrics"
 	"github.com/openshift/cluster-samples-operator/pkg/util"
 )
 
 func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted bool) error {
-	cfg, filePath, doUpsert, updateCfgOnly, err := h.prepSamplesWatchEvent("imagestream", is.Name, is.Annotations, deleted)
-	if cfg != nil && updateCfgOnly {
-		dbg := fmt.Sprintf("clear out removed imagestream %s from progressing/error", is.Name)
-		logrus.Printf("CRDUPDATE %s", dbg)
-		return h.crdwrapper.UpdateStatus(cfg, dbg)
+	cfg, filePath, doUpsert, err := h.prepSamplesWatchEvent("imagestream", is.Name, is.Annotations, deleted)
+	if cfg != nil && util.ConditionTrue(cfg, v1.ImageChangesInProgress) {
+		logrus.Printf("Imagestream %s watch event do upsert %v; no errors in prep %v,  possibly update operator conditions %v", is.Name, doUpsert, err == nil, cfg != nil)
+	} else {
+		logrus.Debugf("Imagestream %s watch event do upsert %v; no errors in prep %v,  possibly update operator conditions %v", is.Name, doUpsert, err == nil, cfg != nil)
 	}
-	logrus.Debugf("Imagestream %s watch event do upsert %v; no errors in prep %v,  possibly update operator conditions %v", is.Name, doUpsert, err == nil, cfg != nil)
 	if cfg != nil {
 		if util.IsUnsupportedArch(cfg) {
 			logrus.Printf("ignoring watch event for imagestream %s ignored because we are on %s",
@@ -42,57 +41,30 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 		if cfg == nil {
 			return nil
 		}
-		cfg = h.refetchCfgMinimizeConflicts(cfg)
 
-		// if we initiated the change, it will be in our cache, and we want to process
-		// the in progress, import error conditions
-		anyChange := false
-		//nonMatchDetail := ""
-		if cache.UpsertsAmount() > 0 {
-			cache.AddReceivedEventFromUpsert(is)
-			if !util.ConditionTrue(cfg, v1.ImageChangesInProgress) || !util.ConditionTrue(cfg, v1.SamplesExist) {
-				logrus.Printf("caching imagestream event %s because we have not yet completed all the samples upserts", is.Name)
-				return nil
-			}
-			if !cache.AllUpsertEventsArrived() {
-				logrus.Printf("caching imagestream event %s because we have not received all %d imagestream events", is.Name, cache.UpsertsAmount())
-				return nil
-			}
-			cache.ClearUpsertsCache()
+		cfg = h.refetchCfgMinimizeConflicts(cfg)
+		cfg, err = h.processImportStatus(is, cfg)
+		if err != nil {
+			logrus.Warningf("process import status error %s on imagestream event %s", err.Error(), is.Name)
+			return err
 		}
 
-		_, skipped := h.skippedImagestreams[is.Name]
-		writeStatus := false
-		dbg := ""
-		cfg = h.refetchCfgMinimizeConflicts(cfg)
-		if util.ConditionTrue(cfg, v1.ImageChangesInProgress) && cache.AllUpsertEventsArrived() {
-			dbg = "clearing in progress with every imagestream reporting in"
-			writeStatus = true
+		// the next two attempts to update samples conditions stem from analysis of installs with some amount of
+		// lower level issues (api server not ready, thottling, watches getting closed) as a failsafe to make sure
+		// the operator reports happiness to the CVO as quickly as possible... ultimately, with so many imagestreams, some of their events
+		// eventually get through
+		if h.shouldSetProgressingFalse(cfg) {
+			dbg := fmt.Sprintf("progressing false update on imagestream %s event", is.Name)
+			logrus.Printf("CRDUPDATE %s", dbg)
 			h.GoodConditionUpdate(cfg, corev1.ConditionFalse, v1.ImageChangesInProgress)
+			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
-		if skipped && util.NameInReason(cfg, util.Condition(cfg, v1.ImportImageErrorsExist).Reason, is.Name) {
-			writeStatus = true
-			if len(dbg) > 0 {
-				dbg = dbg + "; "
-			}
-			dbg = fmt.Sprintf("%sclear %s from import errors", dbg, is.Name)
-			h.clearStreamFromImportError(is.Name, util.Condition(cfg, v1.ImportImageErrorsExist), cfg)
-		}
-		if !skipped {
-			cfg, anyChange = h.processImportStatus(is, cfg)
-			if anyChange {
-				writeStatus = true
-				if len(dbg) > 0 {
-					dbg = dbg + "; "
-				}
-				dbg = fmt.Sprintf("%supdate import errors for %s", dbg, is.Name)
-			}
-		}
-		if writeStatus {
+		if h.shouldSetVersion(cfg) {
+			cfg.Status.Version = h.version
+			dbg := fmt.Sprintf("upd status version to %s on imagestream event %s", h.version, is.Name)
 			logrus.Printf("CRDUPDATE %s", dbg)
 			return h.crdwrapper.UpdateStatus(cfg, dbg)
 		}
-
 		return nil
 
 	}
@@ -121,14 +93,8 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 		is = nil
 	}
 
-	cache.AddUpsert(imagestream.Name)
-
-	if is != nil {
-		is = is.DeepCopy()
-	}
 	err = h.upsertImageStream(imagestream, is, cfg)
 	if err != nil {
-		cache.RemoveUpsert(imagestream.Name)
 		if kerrors.IsAlreadyExists(err) {
 			// it the main loop created, we will let it set the image change reason field
 			return nil
@@ -140,33 +106,34 @@ func (h *Handler) processImageStreamWatchEvent(is *imagev1.ImageStream, deleted
 		return h.crdwrapper.UpdateStatus(cfg, dbg)
 	}
 	// now update progressing condition
-	cfg = h.refetchCfgMinimizeConflicts(cfg)
-	progressing := util.Condition(cfg, v1.ImageChangesInProgress)
-	now := kapis.Now()
-	progressing.LastUpdateTime = now
-	progressing.LastTransitionTime = now
-	logrus.Debugf("Handle changing processing from false to true for imagestream %s", imagestream.Name)
-	progressing.Status = corev1.ConditionTrue
-	if !util.NameInReason(cfg, progressing.Reason, imagestream.Name) {
-		progressing.Reason = progressing.Reason + imagestream.Name + " "
+	cm := &corev1.ConfigMap{}
+	cm.Name = imagestream.Name
+	_, err = h.configmapclientwrapper.Create(cm)
+	if err != nil && !kerrors.IsAlreadyExists(err) {
+		return err
 	}
-	util.ConditionUpdate(cfg, progressing)
-	dbg := fmt.Sprintf("progressing true update for imagestream %s", imagestream.Name)
-	logrus.Printf("CRDUPDATE %s", dbg)
-	return h.crdwrapper.UpdateStatus(cfg, dbg)
 
+	return nil
 }
 
 func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInCluster *imagev1.ImageStream, opcfg *v1.Config) error {
 	// handle jenkins mutations if needed
 	imagestreamInOperatorImage = jenkinsOverrides(imagestreamInOperatorImage)
 
-	// whether we are now skipping this imagestream, or are upserting it, remove any prior errors from the import error
-	// condition; in the skip case, we don't want errors to a now skipped stream blocking availability status; in the upsert
+	// whether we are now skipping this imagestream, or are upserting it, remove any prior import errors;
+	// in the skip case, we don't want errors to a now skipped stream blocking availability status; in the upsert
 	// case, any errors will cause the imagestream controller to attempt another image import
-	h.mapsMutex.Lock()
-	defer h.mapsMutex.Unlock()
-	h.clearStreamFromImportError(imagestreamInOperatorImage.Name, util.Condition(opcfg, v1.ImportImageErrorsExist), opcfg)
+	cm := &corev1.ConfigMap{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      imagestreamInOperatorImage.Name,
+			Namespace: v1.OperatorNamespace,
+		},
+		Data: map[string]string{},
+	}
+	_, err := h.configmapclientwrapper.Update(cm)
+	if err != nil && !kerrors.IsNotFound(err) {
+		return err
+	}
 
 	if _, isok := h.skippedImagestreams[imagestreamInOperatorImage.Name]; isok {
 		if imagestreamInCluster != nil {
@@ -228,7 +195,7 @@ func (h *Handler) upsertImageStream(imagestreamInOperatorImage, imagestreamInClu
 		imagestreamInOperatorImage.Spec.Tags = append(imagestreamInOperatorImage.Spec.Tags, tag)
 	}
 	imagestreamInOperatorImage.ResourceVersion = imagestreamInCluster.ResourceVersion
-	_, err := h.imageclientwrapper.Update(imagestreamInOperatorImage)
+	_, err = h.imageclientwrapper.Update(imagestreamInOperatorImage)
 	if err != nil {
 		return h.processError(opcfg, v1.SamplesExist, corev1.ConditionUnknown, err, "imagestream update error: %v")
 	}
@@ -315,43 +282,15 @@ func (h *Handler) getImporErrorMessage(name string, importError *v1.ConfigCondit
 	return entireMsg
 }
 
-// clearStreamFromImportError assumes the caller has call h.mapsMutex.Lock() and Unlock() appropriately
-func (h *Handler) clearStreamFromImportError(name string, importError *v1.ConfigCondition, cfg *v1.Config) *v1.ConfigCondition {
-	if util.NameInReason(cfg, importError.Reason, name) {
-		logrus.Printf("clearing imagestream %s from the import image error condition", name)
-	}
-	now := kapis.Now()
-	importError.Reason = util.ClearNameInReason(cfg, importError.Reason, name)
-	oldStatus := importError.Status
-	entireMsg := h.getImporErrorMessage(name, importError)
-	if len(entireMsg) > 0 {
-		importError.Message = strings.Replace(importError.Message, entireMsg, "", -1)
-
-	}
-	if len(strings.TrimSpace(importError.Reason)) == 0 {
-		importError.Status = corev1.ConditionFalse
-		importError.Reason = ""
-		importError.Message = ""
-		delete(h.imagestreamRetry, name)
-	} else {
-		importError.Status = corev1.ConditionTrue
-	}
-	if oldStatus != importError.Status {
-		importError.LastTransitionTime = now
-	}
-	importError.LastUpdateTime = now
-	util.ConditionUpdate(cfg, importError)
-	return importError
+func (h *Handler) buildImageStreamErrorMessage() string {
+	msg := fmt.Sprintf("ImageStream import errors have been detected.  Scan the %s namespace for config maps.  Each failing imagestream will have a config map.  And the 'data' map will have entries per image stream tag with details on the errors.",
+		v1.OperatorNamespace)
+	return msg
 }
 
-func (h *Handler) buildImageStreamErrorMessage(name, message string) string {
-	return "<imagestream/" + name + ">" + message + "<imagestream/" + name + ">"
-}
-
-func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (*v1.Config, bool) {
+func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (*v1.Config, error) {
+	var err error
 	anyErrors := false
-	importError := util.Condition(cfg, v1.ImportImageErrorsExist)
-	anyConditionUpdate := false
 	// in case we have to manipulate imagestream retry map
 	h.mapsMutex.Lock()
 	defer h.mapsMutex.Unlock()
@@ -377,6 +316,7 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 			retryIfNeeded = lastRetryTime.Time.Before(tenMinutesAgo)
 		}
 
+		tagsToPotentiallClearFromConfigMap := []string{}
 		for _, statusTag := range is.Status.Tags {
 			// if an error occurred with the latest generation, let's give up as we are no longer "in progress"
 			// in that case as well, but mark the import failure
@@ -398,70 +338,75 @@ func (h *Handler) processImportStatus(is *imagev1.ImageStream, cfg *v1.Config) (
 				if mostRecentErrorGeneration > 0 && mostRecentErrorGeneration >= latestGeneration {
 					logrus.Warningf("Image import for imagestream %s tag %s generation %v failed with detailed message %s", is.Name, statusTag.Tag, mostRecentErrorGeneration, message)
 					anyErrors = true
-					// add this imagestream to the Reason field;
-					// we don't want to initiate imports repeatedly, but we do want to retry periodically as part
-					// of relist
 
 					// if a first time failure, or need to retry
-					if !util.NameInReason(cfg, importError.Reason, is.Name) ||
+					if !h.imageStreamHasErrors(is.Name) ||
 						retryIfNeeded {
 						h.imagestreamRetry[is.Name] = now
-						if !util.NameInReason(cfg, importError.Reason, is.Name) {
-							importError.Reason = importError.Reason + is.Name + " "
-							importError.Message = importError.Message + h.buildImageStreamErrorMessage(is.Name, message)
-						}
-						if importError.Status != corev1.ConditionTrue {
-							importError.Status = corev1.ConditionTrue
-							importError.LastTransitionTime = now
-						}
-						// we always bump last update here as a signal that another import attempt is happening
-						importError.LastUpdateTime = now
-						util.ConditionUpdate(cfg, importError)
-						anyConditionUpdate = true
 						// initiate a retry (if same error happens again, imagestream status does not change, we won't get an event, and we do not try again)
 						imgImport, err := importTag(is, statusTag.Tag)
 						if err != nil {
 							logrus.Warningf("attempted to define and imagestreamimport for imagestream/tag %s/%s but got err %v; simply moving on", is.Name, statusTag.Tag, err)
-							break
 						}
 						if imgImport == nil {
-							break
+							logrus.Warningf("attempted to define an imagestreamimport for imagestream/tag %s/%s bug got a nil image import reference", is.Name, statusTag.Tag)
 						}
-						imgImport, err = h.imageclientwrapper.ImageStreamImports("openshift").Create(context.TODO(), imgImport, kapis.CreateOptions{})
-						if err != nil {
-							logrus.Warningf("attempted to initiate an imagestreamimport retry for imagestream/tag %s/%s but got err %v; simply moving on", is.Name, statusTag.Tag, err)
-							break
+						if imgImport != nil && err == nil {
+							imgImport, err = h.imageclientwrapper.ImageStreamImports("openshift").Create(context.TODO(), imgImport, kapis.CreateOptions{})
+							if err != nil {
+								logrus.Warningf("attempted to initiate an imagestreamimport retry for imagestream/tag %s/%s but got err %v; simply moving on", is.Name, statusTag.Tag, err)
+							}
+							if err == nil {
+								metrics.ImageStreamImportRetry(is.Name)
+								logrus.Printf("initiated an imagestreamimport retry for imagestream/tag %s/%s", is.Name, statusTag.Tag)
+							}
+
 						}
-						metrics.ImageStreamImportRetry(is.Name)
-						logrus.Printf("initiated an imagestreamimport retry for imagestream/tag %s/%s", is.Name, statusTag.Tag)
 
 					}
 
-					// if failed previously, but the message has changed, update it
-					if !strings.Contains(importError.Message, message) {
-						msg := h.getImporErrorMessage(is.Name, importError)
-						importError.Message = strings.Replace(importError.Message, msg, h.buildImageStreamErrorMessage(is.Name, message), -1)
-						anyConditionUpdate = true
+					// update imagestream error message for this tag
+					err = h.storeImageStreamTagError(is.Name, statusTag.Tag, message)
+					if err != nil {
+						return cfg, err
 					}
 
-					util.ConditionUpdate(cfg, importError)
+				} else {
+					// in case the currently observed behavior of imagestreams changes and the conditions array is not
+					// cleared out when a tag is healthy, clear the tag here as well
+					// but at present time, this path will not be hit
+					tagsToPotentiallClearFromConfigMap = append(tagsToPotentiallClearFromConfigMap, statusTag.Tag)
 				}
+			} else {
+				// lack of conditions means there are no errors; conditions seem to now be cleared if errors resolved
+				tagsToPotentiallClearFromConfigMap = append(tagsToPotentiallClearFromConfigMap, statusTag.Tag)
 			}
 		}
+
+		if len(tagsToPotentiallClearFromConfigMap) > 0 {
+			err = h.clearImageStreamTagError(is.Name, tagsToPotentiallClearFromConfigMap)
+		}
+
+	} else {
+		logrus.Debugf("no error/progress checks cause stream %s is skipped", is.Name)
+		// but if skipped, clear out any errors, since we do not care about errors for skipped
+		err := h.configmapclientwrapper.Delete(is.Name)
+		if err != nil && !kerrors.IsNotFound(err) {
+			logrus.Warningf("unexpected error on delete of config map %s: %s", is.Name, err.Error())
+			return cfg, err
+		}
+		return cfg, nil
 	}
 
 	logrus.Debugf("any errors %v for %s", anyErrors, is.Name)
 
-	logrus.Printf("There are no more image imports in flight for imagestream %s", is.Name)
-	cache.RemoveUpsert(is.Name)
-
-	// clear out error for this stream if there were errors previously but no longer are
-	// think a scheduled import failing then recovering
-	if util.NameInReason(cfg, importError.Reason, is.Name) && !anyErrors {
-		logrus.Printf("There are no image import errors for %s", is.Name)
-		h.clearStreamFromImportError(is.Name, importError, cfg)
-		anyConditionUpdate = true
+	if !anyErrors {
+		logrus.Printf("There are no more errors or image imports in flight for imagestream %s", is.Name)
+		err = h.configmapclientwrapper.Delete(is.Name)
+		if kerrors.IsNotFound(err) {
+			err = nil
+		}
 	}
 
-	return cfg, anyConditionUpdate
+	return cfg, err
 }
