@@ -134,6 +134,18 @@ func verifyOperatorUp(t *testing.T) *samplesapi.Config {
 	if err != nil {
 		t.Fatalf("error waiting for samples resource to appear: %v", err)
 	}
+	err = wait.PollImmediate(1*time.Second, 1*time.Minute, func() (bool, error) {
+		_, err = kubeClient.CoreV1().ConfigMaps(samplesapi.OperatorNamespace).Get(
+			context.TODO(), util.IST2ImageMap, metav1.GetOptions{})
+		if err != nil {
+			t.Logf("%s", err.Error())
+			return false, nil
+		}
+		return true, nil
+	})
+	if err != nil {
+		t.Fatalf("error waiting for ist to image map: %s", err.Error())
+	}
 	return cfg
 }
 
@@ -151,6 +163,7 @@ func verifySupportedArch(t *testing.T) bool {
 	}
 	if cfg.Status.Architectures[0] != samplesapi.X86Architecture &&
 		cfg.Status.Architectures[0] != samplesapi.AMDArchitecture &&
+		cfg.Status.Architectures[0] != samplesapi.ARMArchitecture &&
 		cfg.Status.Architectures[0] != samplesapi.S390Architecture &&
 		cfg.Status.Architectures[0] != samplesapi.PPCArchitecture {
 		return false
@@ -442,22 +455,6 @@ func verifyImageStreamsPresent(t *testing.T, content map[string]bool, timeToComp
 	}
 }
 
-func verifyImageChangesInProgress(t *testing.T) {
-	var cfg *samplesapi.Config
-	var err error
-	err = wait.PollImmediate(1*time.Second, 3*time.Minute, func() (bool, error) {
-		cfg, err = crClient.SamplesV1().Configs().Get(context.TODO(), samplesapi.ConfigName, metav1.GetOptions{})
-		if err != nil {
-			t.Logf("%v", err)
-			return false, nil
-		}
-		if util.ConditionTrue(cfg, samplesapi.ImageChangesInProgress) {
-			return true, nil
-		}
-		return false, nil
-	})
-}
-
 func verifyImageStreamsGone(t *testing.T) {
 	time.Sleep(30 * time.Second)
 	content := getSamplesNames(getContentDir(t), nil, t)
@@ -564,9 +561,7 @@ func verifyDeletedImageStreamRecreated(t *testing.T) {
 		cfg := verifyOperatorUp(t)
 		t.Fatalf("error deleting jenkins imagestream %v Config %#v", err, cfg)
 	}
-	// first make sure image changes makes it to true
-	verifyImageChangesInProgress(t)
-	// then make sure the image changes are complete before attempting to fetch deleted IS
+	// make sure the image changes are complete before attempting to fetch deleted IS
 	verifyConditionsCompleteSamplesAdded(t)
 	err = wait.PollImmediate(1*time.Second, 30*time.Second, func() (bool, error) {
 		_, err := imageClient.ImageV1().ImageStreams("openshift").Get(context.TODO(), "jenkins", metav1.GetOptions{})
@@ -946,23 +941,7 @@ func TestSpecManagementStateField(t *testing.T) {
 		t.Fatalf("cfg status mgmt never went to managed %#v", verifyOperatorUp(t))
 	}
 
-	// wait for it to get into pending
-	err = wait.PollImmediate(1*time.Second, 3*time.Minute, func() (bool, error) {
-		cfg, err = crClient.SamplesV1().Configs().Get(context.TODO(), samplesapi.ConfigName, metav1.GetOptions{})
-		if err != nil {
-			t.Logf("%v", err)
-			return false, nil
-		}
-		if util.Condition(cfg, samplesapi.ImageChangesInProgress).LastUpdateTime.After(now.Time) {
-			return true, nil
-		}
-		return false, nil
-	})
-	if err != nil {
-		dumpPod(t)
-		t.Fatalf("error waiting for Config to get into pending: %v samples resource %#v", err, cfg)
-	}
-	// now wait for it to get out of pending
+	// now wait for it to get to steady state
 	err = verifyConditionsCompleteSamplesAdded(t)
 	if err != nil {
 		dumpPod(t)
@@ -1049,10 +1028,7 @@ func TestSkippedProcessing(t *testing.T) {
 	verifySkippedStreamManagedLabel(t, "true")
 	verifySkippedTemplateManagedLabel(t, "true")
 
-	// checking in progress before validating content helps
-	// makes sure we go into image changes true mode from false
-	verifyImageChangesInProgress(t)
-	// then make sure image changes complete
+	// make sure image changes complete
 	err = verifyConditionsCompleteSamplesAdded(t)
 	if err != nil {
 		dumpPod(t)
@@ -1133,14 +1109,28 @@ func coreTestUpgrade(t *testing.T) {
 		t.Fatalf("problem updating deployment env")
 	}
 
-	if cfg.Status.ManagementState == operatorsv1api.Managed {
-		err = wait.PollImmediate(1*time.Second, 3*time.Minute, func() (bool, error) {
-			cfg := verifyOperatorUp(t)
-			if util.ConditionTrue(cfg, samplesapi.MigrationInProgress) {
-				return true, nil
-			}
+	// confirm pod has new version env
+	podClient := kubeClient.CoreV1().Pods(samplesapi.OperatorNamespace)
+	err = wait.PollImmediate(1*time.Second, 3*time.Minute, func() (bool, error) {
+		pods, err := podClient.List(context.TODO(), metav1.ListOptions{})
+		if err != nil {
+			t.Logf("error listing pods: %s", err.Error())
 			return false, nil
-		})
+		}
+		for _, pod := range pods.Items {
+			for _, c := range pod.Spec.Containers {
+				for _, env := range c.Env {
+					if strings.TrimSpace(env.Name) == "RELEASE_VERSION" && env.Value == newVersion {
+						return true, nil
+					}
+				}
+			}
+		}
+		return false, nil
+	})
+	if err != nil {
+		dumpPod(t)
+		t.Fatalf("deployment RELEASE_VERSION change not reflected in pod")
 	}
 
 	if err != nil {
@@ -1161,7 +1151,6 @@ func coreTestUpgrade(t *testing.T) {
 	if cfg.Status.ManagementState == operatorsv1api.Managed {
 		validateContent(t, nil)
 	}
-
 }
 
 func changeMgmtState(t *testing.T, state operatorsv1api.ManagementState) {
